/***********************************************************************************************************
***   GROOVESIZER TB2 - 4 Voice Paraphonic Synth and Step Sequencer
***   for the Arduino DUE with TB2 shield http://groovesizer.com
***   August 2014
***   by MoShang (Jean Marais) - moshang@groovesizer.com
***   Adapted from tutorials by DuaneB at http://rcarduino.blogspot.com
***   And the Groovuino library by Gaétan Ro at http://groovuino.blogspot.com
***   Licensed under a Creative Commons Attribution-ShareAlike 3.0
***   AKWF Waveforms by Adventure Kid is licensed under a Creative Commons Attribution 3.0 Unported License.
***   Based on a work at http://www.adventurekid.se
***   Tillstånd utöver denna licens kan vara tillgängligt från http://www.adventurekid.se.
************************************************************************************************************/
int versionNumber = 67;

#include "DueTimer.h"
#include <MIDI.h>
#include <LiquidCrystal.h>
#include <SdFat.h>

// *** SD CARD ***
// SD chip select pin
const uint8_t chipSelect = SS;
// file system object
SdFat sd;
SdFile file;
char fileName[13];
char folderName[13];
int16_t waveShapeBuffer[600];
uint16_t dirCount = 0; // count how many files are in a directory
uint16_t dirChoice = 999; // nothing is selected yet
uint16_t tempCount = 0;
uint16_t lastDirChoice = 0;
boolean folder = false;
boolean inFolder = false; // are we in a folder?
boolean dirChecked = false; // have we run through the directory to see how many files are inside?
boolean waveshapeLoaded = false;
boolean synthPatchLoaded = false;
int patchBuffer[1900];
boolean saveConfirm = false;
int numberName = 0;
char saveName[13] = {"11.TB2"}; 

// *** LCD ***
// initialize the LCD library with the numbers of the interface pins
LiquidCrystal lcd(12, 11, 5, 4, 3, 2);

// *** BUTTONS ***
#define DEBOUNCE 10  // button debouncer, how many ms to debounce, 5+ ms is usually plenty
// here is where we define the buttons that we'll use. button "1" is the first, button "6" is the 6th, etc
byte buttons[] = {22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35}; // the analog 0-5 pins are also known as 14-19
// This handy macro lets us determine how big the array up above is, by checking the size
#define NUMBUTTONS sizeof(buttons)
// we will track if a button is just pressed, just released, or 'currently pressed'
byte pressed[NUMBUTTONS], justpressed[NUMBUTTONS], justreleased[NUMBUTTONS];

byte lastPressed = 0;

// *** LED ***
#define LED 13
boolean LEDon; // a boolean to keep track of whether the LED should be turned on or off at the end of the loop with updateLED()

// *** POTS ***
int pot[5]; // to store the values of out 5 pots
int potLock[5]; // to lock the pots when they're not being adjusted
unsigned long lockTimer = 0; // keep track oof millis() to see if it's time to lock the pots
int lastPotValue[5]; // keep track of the last value so hte pot doesn't lock while you're adjusting it

// *** SYNTH ***
// the phase accumulator points to the current sample in our wavetable
uint32_t ulPhaseAccumulator[8] = {0, 0, 0, 0, 0, 0, 0, 0};

// the phase increment controls the rate at which we move through the wave table
// higher values = higher frequencies
volatile uint32_t ulPhaseIncrement[8] = {0, 0, 0, 0, 0, 0, 0, 0};   // 32 bit phase increment, see below

// full waveform = 0 to SAMPLES_PER_CYCLE
// Phase Increment for 1 Hz =(SAMPLES_PER_CYCLE_FIXEDPOINT/SAMPLE_RATE) = 1Hz
// Phase Increment for frequency F = (SAMPLES_PER_CYCLE/SAMPLE_RATE)*F
#define SAMPLE_RATE 44100.0
#define SAMPLES_PER_CYCLE 600
#define SAMPLES_PER_CYCLE_FIXEDPOINT (SAMPLES_PER_CYCLE<<20)
#define TICKS_PER_CYCLE (float)((float)SAMPLES_PER_CYCLE_FIXEDPOINT/(float)SAMPLE_RATE)

// to represent 600 we need 10 bits
// Our fixed point format will be 10P22 = 32 bits


// We have 521K flash and 96K ram to play with

// Create a table to hold the phase increments we need to generate midi note frequencies at our 44.1Khz sample rate
#define MIDI_NOTES 128
uint32_t nMidiPhaseIncrement[MIDI_NOTES];

// Create a table to hold pre computed sinewave, the table has a resolution of 600 samples
#define WAVE_SAMPLES 600
// default int is 32 bit, in most cases its best to use uint32_t but for large arrays its better to use smaller
// data types if possible, here we are storing 12 bit samples in 16 bit ints
uint16_t nSineTable[WAVE_SAMPLES];
uint16_t nSquareTable[WAVE_SAMPLES];
uint16_t nSawTable[WAVE_SAMPLES];
uint16_t nTriangleTable[WAVE_SAMPLES];
uint16_t nUserTable1[WAVE_SAMPLES];
uint16_t nUserTable2[WAVE_SAMPLES];
uint16_t nUserTable3[WAVE_SAMPLES];

int16_t pulseWidth = 0;
int16_t uiPulseWidth = 0; // the value set by the user

uint16_t *wavePointer[8];
byte osc1WaveType = 1;
byte osc2WaveType = 1;

byte waveshapes = 8; // how many waveshapes are there? sine, triangle, saw, square, user1, user2, user3, noise

uint32_t ulInput[8];

uint32_t lfoAccumulator = 0;
// the phase increment controls the rate at which we move through the wave table
// higher values = higher frequencies
volatile uint32_t lfoIncrement = 1;   // 32 bit phase increment, see below

int32_t osc1Detune = 0; // range +- 32767
int32_t osc2Detune = 0;
int32_t lastOsc1Detune = 0; // range +- 32767
int32_t lastOsc2Detune = 0;

int16_t osc1Volume = 1023;
int16_t osc2Volume = 1023;

/*
int16_t osc1PanR = 0; // 0 = full left, 1023 = full right
int16_t osc1PanL = 1023 - osc1PanR;
int16_t osc2PanR = 1023;
int16_t osc2PanL = 1023 - osc2PanR;
*/

boolean doOscSpread[8] = {false, false, false, false, false, false, false, false};

// *** FILTER ***

#define FX_SHIFT 8
#define SHIFTED_1 256

int f;
long fb;
int q;
int bufL0, bufL1, hp, bp; // bufR0, bufR1,
unsigned char fType;

boolean filterBypass = true;
int filterCutoff; // filter cutoff
int lastFilterCutoff; // needed so we can switch back to the front-panel keybaord after wind controller
unsigned char filterResonance = 0;

// *** UI ***
boolean splash = true; // show the splash on bootup
boolean uiRefresh = false;
byte mainMenu = 0; // which main mode are we in?
/*
0 = SYNTH
1 = ARPEGGIATOR
2 = SEQUENCER
3 = PREFERENCES
*/
byte menu = 0;
boolean valueChange = false;
/*
0 = SPLASH/MAIN
SYNTH
10 = OSC1
11 = OSC1 - select user waveshape
12 = SQUARE - PULSE WIDTH
20 = OSC2
21 = OSC2 - select user waveshape
22 = SQUARE - PULSE WIDTH
30 = FILTER
40 = ENVELOPE
50 = LFO
60 = MODULATION
70 = SYNTH LOAD
80 = SYNTH SAVE
*/

boolean synPatchLoadSave = 0; //0 or false = load, 1 or true = save

// *** MIDI ***
MIDI_CREATE_INSTANCE(HardwareSerial, Serial1, midiA);
boolean midiMode = false; // are we listening for notes from the onboard keyboard or from external MIDI?
boolean monoMode = false; // best for monomophic controllers (like the Yamaha WX5 Wind MIDI Controller)
byte midiCC[128]; // an array to store the latest incoming MIDI cc values

// *** KEYBOARD ***

int16_t voice[4] = {255, 255, 255, 255}; // 255 means no note is assigned (255 is an impossible MIDI note value)
byte osc1Octave = 4; // can be set by user
byte osc2Octave = 5;
byte osc1OctaveOut; // after modulation
byte osc2OctaveOut;
signed char osc1OctaveMod = 0;
signed char osc2OctaveMod = 0;
byte lastOsc1Octave = 4;
byte lastOsc2Octave = 5;
boolean keyAssigned[13] = {
  false, false, false, false,
  false, false, false, false,
  false, false, false, false,
  false
};
byte voiceCounter; // how many voices are currently used?
byte lastVoiceCount; // we need to be able to know when the last key is released
boolean voiceSounding[4] = {false, false, false, false}; // can we hear this voice?
boolean muteVoice[4] = {false, false, false, false}; // to prevent clicking - is this voice marked to go from true to false for voiceSounding?

// *** ENVELOPE ***
boolean envelopeTrigger = false;
unsigned int attackTime = 50;
unsigned int decayTime = 100;
unsigned int sustainLevel = 800;
unsigned int releaseTime = 500;
int32_t envelopeVolume = 0; // the current volume according to the envelope on a scale from 0 to 1023 (10 bits) - needs to be unsigned so we can multiply with it for modulation
unsigned long attackStartTime = 0;
unsigned long decayStartTime = 0;
unsigned long releaseStartTime = 0;
unsigned char envelopeProgress = 255; // 255 = the envelope is idle
int32_t envOsc1Pitch = 0;
int32_t envOsc2Pitch = 0;
int16_t envOsc1PitchFactor = 0; // value between -1023 and 1023
int16_t envOsc2PitchFactor = 0; // value between -1023 and 1023
int32_t envFilterCutoff = 0;
int16_t envFilterCutoffFactor = 0; // value between -1023 and 1023
int16_t envLfoRate = 0;

// *** LFO ***
uint16_t *lfoShapePointer = &nSineTable[0]; // pointer to the array for the waveshape we're using for the LFO
int16_t tmpLFO = 0;
byte lfoShape = 0;
uint16_t lfoRate = 50;
uint16_t userLfoRate = 50;
boolean lfoAdvance = false;
uint16_t lfoAmount = 0; // value between 0 and 1023
int16_t tmpCutoff = 0;
uint16_t lfoIndex = 0;
boolean lfoLowRange = true;
boolean retrigger = true;
int32_t lfoOsc1Detune = 0;
int32_t lfoOsc2Detune = 0;
int16_t lfoOsc1DetuneFactor = 0;
int16_t lfoOsc2DetuneFactor = 0;
int32_t lfoAmp = 0;
int32_t lfoAmpFactor = 0; // 0 to 1023
int16_t lfoPwFactor = 0;
byte dest = 0; // how many destinatiosn for a given source?
boolean lfoLED = false; // do we want to send the current LFO value to the LED?

// *** MODULATION ***
byte source = 0; // 0 = LFO, 1 = ENVELOPE, 2 = VELOCITY, 3 = USER CC ()
byte destination = 0;
/*
LFO
0 = Osc1 pitch
1 = Osc2 pitch
2 = Osc1 octave
3 = Osc2 octave
4 = Filter Cutoff
5 = AMP
6 = Pulse Width

ENVELOPE
7 = Osc1 pitch
8 = Osc2 pitch
9 = Filter Cutoff
10 = LFO rate
11 = LFO to Filter Cutoff

*/


